#ifndef UART_H
#define UART_H
/************************************************************************
Title:    C include file for the UART library (uart.c)
Author:   Peter Fleury <pfleury@gmx.ch>   http://jump.to/fleury
Date:     1-MAY-2002
Software: AVR-GCC 3.0 with GNU binutils
Hardware: any AVR with built-in UART, tested on AT90S8515 at 4 Mhz

DESCRIPTION:
    An interrupt is generated when the UART has finished transmitting or
    receiving a byte. The interrupt handling routines use circular buffers
    for buffering received and transmitted data.
    
    The UART_RX_BUFFER_SIZE and UART_TX_BUFFER_SIZE variables define
    the buffer size in bytes. Note that these variables must be a 
    power of 2.
    
USAGE:
    Refere to the header file uart.h for a description of the routines. 
    See also example test_uart.c.

NOTES:
    Based on Atmel Application Note AVR306
          
*************************************************************************/

/*
** constants and macros
*/
#define UART_BAUD_SELECT(baudRate,xtalCpu) ((xtalCpu)/((baudRate)*16l)-1)

#ifndef P
#define P(s) ({static const char c[] __attribute__ ((progmem)) = s;c;})
#endif


/* 
** high byte error return code of uart_getc()
*/

#define UART_FRAME_ERROR      ((BV(FE))  << 8)   /* Framing Error by UART       */
#define UART_OVERRUN_ERROR    ((BV(OVR)) << 8)   /* Overrun condition by UART   */
#define UART_BUFFER_OVERFLOW  0x0200             /* receive ringbuffer overflow */
#define UART_NO_DATA          0x0100             /* no receive data available   */


/*
** function prototypes
*/

/*************************************************************************
Function: uart_init()
Purpose:  initialize UART and set baudrate
Input:    baudrate using macro UART_BAUD_SELECT()
Returns:  none
**************************************************************************/
extern void uart_init(unsigned char baudrate);


/*************************************************************************
Function: uart_getc()
Purpose:  return byte from ringbuffer  
Returns:  lower byte:  received byte from ringbuffer
          higher byte: last receive error
**************************************************************************/
extern unsigned int uart_getc(void);


/*************************************************************************
Function: uart_putc()
Purpose:  write byte to ringbuffer for transmitting via UART
Input:    byte to be transmitted
Returns:  none          
**************************************************************************/
extern void uart_putc(unsigned char data);


/*************************************************************************
Function: uart_puts()
Purpose:  transmit string to UART
Input:    string to be transmitted
Returns:  none          
**************************************************************************/
extern void uart_puts(const char *s );


/*************************************************************************
Function: uart_puts_p()
Purpose:  transmit string from program memory to UART
Input:    program memory string to be transmitted
Returns:  none
**************************************************************************/
extern void uart_puts_p(const char *s );
#define uart_puts_P(__s)       uart_puts_p(P(__s))


#endif // UART_H 

